package core.tasks;

import core.entities.User;
import core.entities.timemanager.Task;
import core.entities.timemanager.Taskgroup;
import core.repositories.timemanager.TaskRepository;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.orm.jpa.DataJpaTest;
import org.springframework.boot.test.autoconfigure.orm.jpa.TestEntityManager;
import org.springframework.test.context.jdbc.Sql;
import org.springframework.test.context.jdbc.SqlGroup;
import org.springframework.test.context.junit.jupiter.SpringExtension;

import java.util.List;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertTrue;


@ExtendWith(SpringExtension.class)
@DataJpaTest
public class TaskRepositoryTest {

    @Autowired private TaskRepository taskRepository;
    @Autowired private TestEntityManager testEntityManager;

    @Test
    @SqlGroup({
            @Sql("classpath:taskgroupRepositoryTestEntries.sql"),
            @Sql("classpath:taskRepositoryEntries.sql")
    })
    void findAllByUser() {
        User testUser1 = testEntityManager.find(User.class, 1L);
        User testUser2 = testEntityManager.find(User.class, 2L);

        //Situation 1: Non existent user
        assertEquals(0, taskRepository.findAllByUser("Rotzbakke", true).size());
        assertEquals(0, taskRepository.findAllByUser("Rotzbakke", false).size());

        //Situation 2: User with no tasks at all
        assertEquals(0, taskRepository.findAllByUser(testUser2.getUsername(), false).size());
        assertEquals(0, taskRepository.findAllByUser(testUser2.getUsername(), true).size());

        //Situation 3: User with finished and unfinished tasks
        List<Task> tasks_user1_false = taskRepository.findAllByUser(testUser1.getUsername(), false);
        List<Task> tasks_user1_true = taskRepository.findAllByUser(testUser1.getUsername(), true);
        assertEquals(7, tasks_user1_true.size());
        assertEquals(7, tasks_user1_false.size());

        assertTrue(tasks_user1_false.contains(testEntityManager.find(Task.class, 1L)));
        assertTrue(tasks_user1_false.contains(testEntityManager.find(Task.class, 3L)));
        assertTrue(tasks_user1_false.contains(testEntityManager.find(Task.class, 5L)));
        assertTrue(tasks_user1_false.contains(testEntityManager.find(Task.class, 7L)));
        assertTrue(tasks_user1_false.contains(testEntityManager.find(Task.class, 9L)));
        assertTrue(tasks_user1_false.contains(testEntityManager.find(Task.class, 11L)));
        assertTrue(tasks_user1_false.contains(testEntityManager.find(Task.class, 12L)));

        assertTrue(tasks_user1_true.contains(testEntityManager.find(Task.class, 2L)));
        assertTrue(tasks_user1_true.contains(testEntityManager.find(Task.class, 4L)));
        assertTrue(tasks_user1_true.contains(testEntityManager.find(Task.class, 6L)));
        assertTrue(tasks_user1_true.contains(testEntityManager.find(Task.class, 8L)));
        assertTrue(tasks_user1_true.contains(testEntityManager.find(Task.class, 10L)));
        assertTrue(tasks_user1_true.contains(testEntityManager.find(Task.class, 13L)));
        assertTrue(tasks_user1_true.contains(testEntityManager.find(Task.class, 14L)));
    }
}
